#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "nokia5110.h"
#include "DS3231.h"
#include "simple_twi.h"
#include "dht.h"
#include <math.h>
#define DHT11_PIN 3

volatile int set_hour;
volatile int set_min;
volatile int alarm = 0;
volatile int is_confirmed = 0;

/* Intrerupere pe butonul PB0 pentru setare alarma */
ISR(PCINT1_vect)
{
	char data[5];
	if ((PINB & (1 << PB0)) == 0)
	{
		int nrhours = 0;
		int nrmin = 0;
		alarm = 1;
		is_confirmed = 0;
		
		//cat timp nu s-a apasat confirmare alarma
		while(!((PINB & (1 << PB3)) == 0) && is_confirmed == 0) {
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			if ((PINB & (1 << PB1)) == 0) //setare ora
			{
				nrhours++;
				if (nrhours == 24)
					nrhours = 0;
				_delay_ms(200);
			}
			
			if ((PINB & (1 << PB2)) == 0) //setare minute
			{
				nrmin++;
				if(nrmin == 60)
					nrmin = 0;
				_delay_ms(200);
			}
			
			if(nrhours < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrhours, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_write_string(" : ", 2);
			if(nrmin < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrmin, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_render();
		}
		
		//s-a apasat confirmare alarma
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("The alarm is", 1);
		nokia_lcd_set_cursor(36, 25);
		nokia_lcd_write_string("set", 1);
		nokia_lcd_render();
		_delay_ms(500);
		nokia_lcd_clear();
		
		set_hour = nrhours;
		set_min = nrmin;
		is_confirmed = 1;
	}
}

/* Intrerupere pe butonul PD6 pentru setare ora curenta */
ISR(PCINT3_vect)
{
	char data[5];
	if ((PIND & (1 << PD6)) == 0)
	{
		int nrhours = 0;
		int nrmin = 0;
		
		//cat timp nu s-a apasat confirmare ora setata
		while(!((PINB & (1 << PB3)) == 0)) {
			nokia_lcd_clear();
			nokia_lcd_set_cursor(0, 15);
			if ((PINB & (1 << PB1)) == 0) //setare ora
			{
				nrhours++;
				if (nrhours == 24)
					nrhours = 0;
				_delay_ms(200);
			}
			
			if ((PINB & (1 << PB2)) == 0) //setare minute
			{
				nrmin++;
				if(nrmin == 60)
					nrmin = 0;
				_delay_ms(200);
			}
			
			if(nrhours < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrhours, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_write_string(" : ", 2);
			if(nrmin < 10)
				nokia_lcd_write_string("0", 2);
			itoa(nrmin, data, 10);
			nokia_lcd_write_string(data, 2);
			nokia_lcd_render();
		}
		
		//s-a apasat confirmare ora
		uint8_t nrs, nrm, nrh, day1, day2, month, year;
		DS3231_getTime(&nrs, &nrm, &nrh, &day1, &day2, &month, &year);
		DS3231_setTime(0, nrmin, nrhours, day1, day2, month, year);
		
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("The clock is", 1);
		nokia_lcd_set_cursor(36, 25);
		nokia_lcd_write_string("set", 1);
		nokia_lcd_render();
		_delay_ms(500);
		nokia_lcd_clear();
	}
}


/* initializeaza intrarile si iesirile */
void IO_init()
{
	
	// butoanele sunt puse ca intrare, cu rezistenta de pull-up
	//buton setare alarma
	DDRB &= ~(1 << PB0);
	PORTB |= (1 << PB0);

	//buton setare ora pentru alarma
	DDRB &= ~(1 << PB1);
	PORTB |= (1 << PB1);
	
	//buton setare minute pentru alarma
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);
	
	//buton oprire alarma
	DDRB &= ~(1 << PB3);
	PORTB |= (1 << PB3);
	
	//buton setare ora
	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);
	
	//
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);

	// Activez PIN CHANGE INTERRUPT ENABLE pentru PCINT15:8 si 31:24
	PCICR |= (1 << PCIE1) | (1 << PCIE3);
	// Activez intreruperea pe butoanele PB0 si PD6
	PCMSK1 |= (1 << PCINT8);
	PCMSK3 |= (1 << PCINT30);
}


/* Genereaza sunetul pentru tipul de ton folosit: */
void speaker_morse(int tip_morse) {
	int i;

	/* Alternam starea pinului PD4 la fiecare milisecunda. */
	for (i = 0; i < tip_morse; ++i) {
		PIND = (1 << PD4);

		_delay_ms(1);
	}
}

/* ora curenta este cea setata in alarma */
void alarm_ready(void) {

	/* Setez pinul PD4 ca iesire. */
	DDRD |= (1 << PD4);

	/* Opresc speaker-ul. */
	PORTD &= ~(1 << PD4);
	
	/* activez butonul de light al display-ului */
	DDRC |= (1 << PC7);
	PORTC &= ~(1 << PC7);
	
	/* cat timp alarma nu este oprita */
	while (!((PINB & (1 << PB3)) == 0) && alarm == 1) {
		speaker_morse(100);
		PORTC |= (1 << PC7);
		_delay_ms(50);
		PORTC &= ~(1 << PC7);
		
		nokia_lcd_clear();
		nokia_lcd_set_cursor(0, 15);
		nokia_lcd_write_string("Wake Up!", 2);
		nokia_lcd_render();
	}
	
	PORTC |= (1 << PC7);	
	alarm = 0;
}

void print_month(int month)
{
	switch(month) {
		case 1:
			nokia_lcd_write_string("JAN", 1);
			break;
		case 2:
			nokia_lcd_write_string("FEB", 1);
			break;
		case 3:
			nokia_lcd_write_string("MAR", 1);
			break;
		case 4:
			nokia_lcd_write_string("AVR", 1);
			break;
		case 5:
			nokia_lcd_write_string("MAY", 1);
			break;
		case 6:
			nokia_lcd_write_string("JUN", 1);
			break;
		case 7:
			nokia_lcd_write_string("JUL", 1);
			break;
		case 8:
			nokia_lcd_write_string("AUG", 1);
			break;
		case 9:
			nokia_lcd_write_string("SEP", 1);
			break;
		case 10:
			nokia_lcd_write_string("OCT", 1);
			break;
		case 11:
			nokia_lcd_write_string("NOV", 1);
			break;
		case 12:
			nokia_lcd_write_string("DEC", 1);
			break;
		default:
			break;
	}
}

int main(void)
{
	char data[5];
	uint8_t nrs,min,hours,day1,day2,month,year;
	
	int8_t temperature = 0;
	int8_t hummidity = 0;

	char temp_msg[30];
	char humm_msg[30];
	
	twi_INIT();
	nokia_lcd_init();
	nokia_lcd_clear();
	
	/* activare intreruperi */
	sei();
	IO_init();
	_delay_ms(500);
	
	while(1)
	{
		
		nokia_lcd_clear();
		DS3231_getTime(&nrs, &min, &hours, &day1, &day2, &month, &year);
		
		if (min == set_min && hours == set_hour && alarm == 1) {
			alarm_ready();
			nokia_lcd_clear();
		}
		
		/* afisare ora */
		if (hours <= 9)
			nokia_lcd_write_string("0", 2);
		itoa(hours,data,10);
		nokia_lcd_write_string(data, 2);
		nokia_lcd_write_string(":", 2);
		if (min <= 9)
			nokia_lcd_write_string("0", 2);
		itoa(min,data,10);
		nokia_lcd_write_string(data, 2);
		
		/* ledul user va sta o secunda aprins si o secunda stins */
		if (nrs % 2 == 0)
			PORTD |= (1 << PD7);
		else
			PORTD &= ~(1 << PD7);
		
		/* afisare zi curenta */
		nokia_lcd_set_cursor(0, 20);
		if (day2 <= 9)
		nokia_lcd_write_string("0", 1);
		itoa(day2,data,10);
		nokia_lcd_write_string(data, 1);
		nokia_lcd_write_string("/", 1);
		print_month(month);
		nokia_lcd_write_string("/", 1);
		itoa(year,data,10);
		nokia_lcd_write_string(data, 1);
		
		/* preluare date de la senzor */
		dht_gettemperaturehumidity(&temperature, &hummidity);
		_delay_ms(100);
		
		/* afisare temperatura si umiditate */
		memset(temp_msg, 0, sizeof(temp_msg));
		sprintf(temp_msg, "Temp: %d C", (int)temperature);
		
		memset(humm_msg, 0, sizeof(humm_msg));
		sprintf(humm_msg, "H: %d %%", (int)hummidity);
		
		nokia_lcd_set_cursor(0, 30);
		nokia_lcd_write_string(temp_msg, 1);
		
		nokia_lcd_set_cursor(0, 40);
		nokia_lcd_write_string(humm_msg, 1);

		nokia_lcd_render();
		
		_delay_ms(500);
		
	}
	
}
